@@ -20,7 +20,11 @@ from pymupdf import Font
 from babeldoc import asynchronize
 from babeldoc.assets.assets import warmup
 from babeldoc.babeldoc_exception.BabelDOCException import ExtractTextError
+from babeldoc.babeldoc_exception.BabelDOCException import (
+    InputFileGeneratedByBabelDOCError,
+)
 from babeldoc.const import CACHE_FOLDER
+from babeldoc.const import WATERMARK_VERSION
 from babeldoc.format.pdf.converter import TranslateConverter
 from babeldoc.format.pdf.document_il import il_version_1
 from babeldoc.format.pdf.document_il.backend.pdf_creater import SAVE_PDF_STAGE_NAME
@@ -97,6 +101,59 @@ def safe_save(doc, *args, **kwargs):
         doc.ez_save(*args, **kwargs)
 
 
+def check_metadata(pdf: Document):
+    meta = pdf.metadata
+    if not meta:
+        return
+    producer = meta.get("producer", None)
+    if (
+        producer
+        and "BabelDOC" in producer
+        and "Translation_generated_by_AI,please_carefully_discern" in producer
+    ):
+        raise InputFileGeneratedByBabelDOCError(
+            "Input file is generated by BabelDOC, Cannot translate files that have already been translated."
+        )
+
+
+def add_metadata(
+    translate_result: TranslateResult, translate_config: TranslationConfig
+):
+    processed = []
+    for attr in (
+        "mono_pdf_path",
+        "dual_pdf_path",
+        "no_watermark_mono_pdf_path",
+        "no_watermark_dual_pdf_path",
+    ):
+        path = getattr(translate_result, attr)
+        if not path or path in processed:
+            continue
+        processed.append(path)
+
+        temp_path = translate_config.get_working_file_path(f"{path.stem}.cmap.pdf")
+        pdf = pymupdf.open(path)
+        meta = pdf.metadata
+        if not meta:
+            meta = {}
+        creator = meta.get("creator", None)
+        producer = meta.get("producer", None)
+        if producer:
+            if not creator:
+                creator = producer
+            else:
+                creator += f", {producer}"
+
+        translated_by = f"BabelDOC{WATERMARK_VERSION}_{time.time()}_Translation_generated_by_AI,please_carefully_discern"
+        if translate_config.metadata_extra_data:
+            translated_by += f"_{translate_config.metadata_extra_data}"
+        meta["producer"] = translated_by
+        meta["creator"] = creator
+        pdf.set_metadata(meta)
+        safe_save(pdf, temp_path)
+        shutil.move(temp_path, path)
+
+
 def fix_cmap(translate_result: TranslateResult, translate_config: TranslationConfig):
     processed = []
     for attr in (
@@ -127,6 +184,19 @@ def verify_file_hash(file_path: str, expected_hash: str) -> bool:
     return sha256_hash.hexdigest() == expected_hash
 
 
+def translator_supports_llm(translator) -> bool:
+    if not translator or not hasattr(translator, "do_llm_translate"):
+        return False
+    try:
+        translator.do_llm_translate(None)
+        return True
+    except NotImplementedError:
+        return False
+    except Exception as exc:  # pragma: no cover - defensive logging
+        logger.debug("translator %s failed llm detection: %s", translator, exc)
+        return False
+
+
 def start_parse_il(
     inf: BinaryIO,
     pages: list[int] | None = None,
@@ -255,12 +325,32 @@ def get_translation_stage(
 ) -> list[tuple[str, float]]:
     result = copy.deepcopy(TRANSLATE_STAGES)
     should_remove = []
-    if not translation_config.table_model:
-        should_remove.append(TableParser.stage_name)
-    if translation_config.skip_scanned_detection:
-        should_remove.append(DetectScannedFile.stage_name)
-    if not translation_config.auto_extract_glossary:
-        should_remove.append(AutomaticTermExtractor.stage_name)
+
+    # If only parsing and generating PDF, skip all translation-related stages
+    if translation_config.only_parse_generate_pdf:
+        should_remove.extend(
+            [
+                DetectScannedFile.stage_name,
+                LayoutParser.stage_name,
+                TableParser.stage_name,
+                ParagraphFinder.stage_name,
+                StylesAndFormulas.stage_name,
+                AutomaticTermExtractor.stage_name,
+                ILTranslator.stage_name,
+                Typesetting.stage_name,
+            ]
+        )
+    else:
+        # Original logic for selective removal
+        if not translation_config.table_model:
+            should_remove.append(TableParser.stage_name)
+        if translation_config.skip_scanned_detection:
+            should_remove.append(DetectScannedFile.stage_name)
+        if not translation_config.auto_extract_glossary:
+            should_remove.append(AutomaticTermExtractor.stage_name)
+        if translation_config.skip_translation:
+            should_remove.append(ILTranslator.stage_name)
+
     result = [x for x in result if x[0] not in should_remove]
     return result
 
@@ -456,7 +546,6 @@ def fix_filter(doc):
         if f[0] == "xref":
             data = doc.xref_stream(page_piece)
             doc.update_stream(page_piece, data)
-
     for page in doc:
         contents = page.get_contents()
         if len(contents) > 1:
@@ -465,6 +554,38 @@ def fix_filter(doc):
             doc.update_object(r, "<<>>")
             doc.update_stream(r, b" ".join(page_streams))
             doc.xref_set_key(page.xref, "Contents", f"{r} 0 R")
+    return
+    # skip rotate for now
+    for page in doc:
+        contents = page.get_contents()
+        t, v = doc.xref_get_key(page.xref, "Rotate")
+        rotate = -int(v) if t == "int" else 0
+        if len(contents) > 1 or rotate:
+            page_streams = [doc.xref_stream(i) for i in contents]
+            r = doc.get_new_xref()
+            page_prefix = b""
+            page_suffix = b""
+            if rotate:
+                m0 = pymupdf.Matrix(rotate)
+                b0 = page.mediabox * m0
+                m1 = m0 * pymupdf.Matrix(1, 0, 0, 1, b0.x0, -b0.y0)
+                page_prefix = (
+                    f" {m1.a} {m1.b} {m1.c} {m1.d} {m1.e} {m1.f} cm q ".encode()
+                )
+                page_suffix = b" Q "
+                update_page_bbox(doc, page, page.cropbox * m1, "CropBox")
+                update_page_bbox(doc, page, page.mediabox * m1, "MediaBox")
+                update_page_bbox(doc, page, page.artbox * m1, "ArtBox")
+                update_page_bbox(doc, page, page.bleedbox * m1, "BleedBox")
+                doc.xref_set_key(page.xref, "Rotate", "0")
+            doc.update_object(r, "<<>>")
+            doc.update_stream(r, page_prefix + b" ".join(page_streams) + page_suffix)
+            doc.xref_set_key(page.xref, "Contents", f"{r} 0 R")
+
+
+def update_page_bbox(doc, page, box, key):
+    if doc.xref_get_key(page.xref, key)[0] == "array":
+        doc.xref_set_key(page.xref, key, f"[{box.x0} {box.y0} {box.x1} {box.y1}]")
 
 
 def do_translate(
@@ -474,6 +595,15 @@ def do_translate(
         translation_config.progress_monitor = pm
         original_pdf_path = translation_config.input_file
         logger.info(f"start to translate: {original_pdf_path}")
+        try:
+            check_metadata(Document(original_pdf_path))
+        except InputFileGeneratedByBabelDOCError as e:
+            logger.error(
+                f"input file {original_pdf_path} is generated by BabelDOC, Cannot translate files that have already been translated."
+            )
+            raise e
+        except Exception as e:
+            logger.warning(f"Error in check metadata, continue: {e}")
         start_time = time.time()
         peak_memory_usage = 0
         with MemoryMonitor() as memory_monitor:
@@ -626,6 +756,7 @@ def do_translate(
         result.peak_memory_usage = peak_memory_usage
 
         fix_cmap(result, translation_config)
+        add_metadata(result, translation_config)
         try:
             migrate_toc(translation_config, result)
         except Exception as e:
@@ -652,6 +783,9 @@ def do_translate(
 def migrate_toc(
     translation_config: TranslationConfig, translate_result: TranslateResult
 ):
+    if translation_config.use_alternating_pages_dual:
+        logger.info('skipping TOC migration for "use_alternating_pages_dual" mode')
+        return
     old_doc = Document(translation_config.input_file)
     if not old_doc:
         return
@@ -840,6 +974,15 @@ def _do_translate_single(
     if check_cid_char(docs):
         raise ExtractTextError("The document contains too many CID chars.")
 
+    # Skip all translation processing if only_parse_generate_pdf is enabled
+    if translation_config.only_parse_generate_pdf:
+        logger.debug("only_parse_generate_pdf enabled, skipping translation processing")
+        # Skip directly to PDF generation
+        pdf_creater = PDFCreater(temp_pdf_path, docs, translation_config, mediabox_data)
+        result = pdf_creater.write(translation_config)
+        result.original_pdf_path = translation_config.input_file
+        return result
+
     # Rest of the original translation logic...
     # [Previous implementation of do_translate continues here]
 
@@ -848,7 +991,9 @@ def _do_translate_single(
         logger.debug("skipping scanned file detection")
     else:
         logger.debug("start detect scanned file")
-        DetectScannedFile(translation_config).process(docs)
+        DetectScannedFile(translation_config).process(
+            docs, temp_pdf_path, mediabox_data
+        )
         logger.debug("finish detect scanned file")
         if translation_config.debug:
             xml_converter.write_json(
@@ -890,26 +1035,28 @@ def _do_translate_single(
         )
 
     translate_engine = translation_config.translator
+    term_extraction_engine = translation_config.get_term_extraction_translator()
 
-    support_llm_translate = False
-    try:
-        if translate_engine and hasattr(translate_engine, "do_llm_translate"):
-            translate_engine.do_llm_translate(None)
-            support_llm_translate = True
-    except NotImplementedError:
-        support_llm_translate = False
+    support_llm_translate = translator_supports_llm(translate_engine)
+    support_llm_term_extraction = translator_supports_llm(term_extraction_engine)
 
-    if support_llm_translate and translation_config.auto_extract_glossary:
-        AutomaticTermExtractor(translate_engine, translation_config).procress(docs)
+    if support_llm_term_extraction and translation_config.auto_extract_glossary:
+        AutomaticTermExtractor(term_extraction_engine, translation_config).procress(
+            docs
+        )
 
-    if support_llm_translate:
-        il_translator = ILTranslatorLLMOnly(translate_engine, translation_config)
+    if not translation_config.skip_translation:
+        if support_llm_translate:
+            il_translator = ILTranslatorLLMOnly(translate_engine, translation_config)
+        else:
+            il_translator = ILTranslator(translate_engine, translation_config)
+
+        il_translator.translate(docs)
+        del il_translator
+        logger.debug(f"finish ILTranslator from {temp_pdf_path}")
     else:
-        il_translator = ILTranslator(translate_engine, translation_config)
+        logger.info("skip ILTranslator")
 
-    il_translator.translate(docs)
-    del il_translator
-    logger.debug(f"finish ILTranslator from {temp_pdf_path}")
     if translation_config.debug:
         xml_converter.write_json(
             docs,
