@@ -22,7 +22,7 @@ from babeldoc.translator.translator import OpenAITranslator
 from babeldoc.translator.translator import set_translate_rate_limiter
 
 logger = logging.getLogger(__name__)
-__version__ = "0.4.20"
+__version__ = "0.5.11"
 
 
 def create_parser():
@@ -71,6 +71,18 @@ def create_parser():
         "--rpc-doclayout4",
         help="RPC service host address for document layout analysis",
     )
+    parser.add_argument(
+        "--rpc-doclayout5",
+        help="RPC service host address for document layout analysis",
+    )
+    parser.add_argument(
+        "--rpc-doclayout6",
+        help="RPC service host address for document layout analysis",
+    )
+    parser.add_argument(
+        "--rpc-doclayout7",
+        help="RPC service host address for document layout analysis",
+    )
     parser.add_argument(
         "--generate-offline-assets",
         default=None,
@@ -86,6 +98,11 @@ def create_parser():
         default=None,
         help="Working directory for translation. If not set, use temp directory.",
     )
+    parser.add_argument(
+        "--metadata-extra-data",
+        default=None,
+        help="Extra data for metadata",
+    )
     # translation option argument group
     translation_group = parser.add_argument_group(
         "Translation",
@@ -286,6 +303,67 @@ def create_parser():
         default=False,
         help="Save automatically extracted glossary terms to a CSV file in the output directory.",
     )
+    translation_group.add_argument(
+        "--disable-graphic-element-process",
+        action="store_true",
+        default=False,
+        help="Disable graphic element process. (default: False)",
+    )
+    translation_group.add_argument(
+        "--no-merge-alternating-line-numbers",
+        action="store_false",
+        dest="merge_alternating_line_numbers",
+        default=True,
+        help="Disable post-processing that merges alternating line-number layouts (by default this feature is enabled).",
+    )
+    translation_group.add_argument(
+        "--skip-translation",
+        action="store_true",
+        default=False,
+        help="Skip translation step. (default: False)",
+    )
+    translation_group.add_argument(
+        "--skip-form-render",
+        action="store_true",
+        default=False,
+        help="Skip form rendering. (default: False)",
+    )
+    translation_group.add_argument(
+        "--skip-curve-render",
+        action="store_true",
+        default=False,
+        help="Skip curve rendering. (default: False)",
+    )
+    translation_group.add_argument(
+        "--only-parse-generate-pdf",
+        action="store_true",
+        default=False,
+        help="Only parse PDF and generate output PDF without translation (default: False). This skips all translation-related processing including layout analysis, paragraph finding, style processing, and translation itself.",
+    )
+    translation_group.add_argument(
+        "--remove-non-formula-lines",
+        action="store_true",
+        default=False,
+        help="Remove non-formula lines from paragraph areas. This removes decorative lines that are not part of formulas, while protecting lines in figure/table areas. (default: False)",
+    )
+    translation_group.add_argument(
+        "--non-formula-line-iou-threshold",
+        type=float,
+        default=0.9,
+        help="IoU threshold for detecting paragraph overlap when removing non-formula lines. Higher values are more conservative. (default: 0.9)",
+    )
+    translation_group.add_argument(
+        "--figure-table-protection-threshold",
+        type=float,
+        default=0.9,
+        help="IoU threshold for protecting lines in figure/table areas when removing non-formula lines. Higher values provide more protection. (default: 0.9)",
+    )
+    translation_group.add_argument(
+        "--skip-formula-offset-calculation",
+        action="store_true",
+        default=False,
+        help="Skip formula offset calculation (default: False)",
+    )
     # service option argument group
     service_group = translation_group.add_mutually_exclusive_group()
     service_group.add_argument(
@@ -311,6 +389,39 @@ def create_parser():
         "-k",
         help="The API key for the OpenAI API.",
     )
+    service_group.add_argument(
+        "--openai-term-extraction-model",
+        default=None,
+        help="OpenAI model to use for automatic term extraction. Defaults to --openai-model when unset.",
+    )
+    service_group.add_argument(
+        "--openai-term-extraction-base-url",
+        default=None,
+        help="Base URL for the OpenAI API used during automatic term extraction. Falls back to --openai-base-url when unset.",
+    )
+    service_group.add_argument(
+        "--openai-term-extraction-api-key",
+        default=None,
+        help="API key for the OpenAI API used during automatic term extraction. Falls back to --openai-api-key when unset.",
+    )
+    service_group.add_argument(
+        "--enable-json-mode-if-requested",
+        action="store_true",
+        default=False,
+        help="Enable JSON mode for OpenAI requests.",
+    )
+    service_group.add_argument(
+        "--send-dashscope-header",
+        action="store_true",
+        default=False,
+        help="Send DashScope data inspection header to disable input/output inspection.",
+    )
+    service_group.add_argument(
+        "--no-send-temperature",
+        action="store_true",
+        default=False,
+        help="Do not send temperature parameter to OpenAI API (default: send temperature).",
+    )
 
     return parser
 
@@ -358,7 +469,27 @@ async def main():
             base_url=args.openai_base_url,
             api_key=args.openai_api_key,
             ignore_cache=args.ignore_cache,
+            enable_json_mode_if_requested=args.enable_json_mode_if_requested,
+            send_dashscope_header=args.send_dashscope_header,
+            send_temperature=not args.no_send_temperature,
         )
+        term_extraction_translator = translator
+        if (
+            args.openai_term_extraction_model
+            or args.openai_term_extraction_base_url
+            or args.openai_term_extraction_api_key
+        ):
+            term_extraction_translator = OpenAITranslator(
+                lang_in=args.lang_in,
+                lang_out=args.lang_out,
+                model=args.openai_term_extraction_model or args.openai_model,
+                base_url=(args.openai_term_extraction_base_url or args.openai_base_url),
+                api_key=args.openai_term_extraction_api_key or args.openai_api_key,
+                ignore_cache=args.ignore_cache,
+                enable_json_mode_if_requested=args.enable_json_mode_if_requested,
+                send_dashscope_header=args.send_dashscope_header,
+                send_temperature=not args.no_send_temperature,
+            )
     else:
         raise ValueError("Invalid translator type")
 
@@ -381,6 +512,18 @@ async def main():
         from babeldoc.docvision.rpc_doclayout4 import RpcDocLayoutModel
 
         doc_layout_model = RpcDocLayoutModel(host=args.rpc_doclayout4)
+    elif args.rpc_doclayout5:
+        from babeldoc.docvision.rpc_doclayout5 import RpcDocLayoutModel
+
+        doc_layout_model = RpcDocLayoutModel(host=args.rpc_doclayout5)
+    elif args.rpc_doclayout6:
+        from babeldoc.docvision.rpc_doclayout6 import RpcDocLayoutModel
+
+        doc_layout_model = RpcDocLayoutModel(host=args.rpc_doclayout6)
+    elif args.rpc_doclayout7:
+        from babeldoc.docvision.rpc_doclayout7 import RpcDocLayoutModel
+
+        doc_layout_model = RpcDocLayoutModel(host=args.rpc_doclayout7)
     else:
         from babeldoc.docvision.doclayout import DocLayoutModel
 
@@ -478,6 +621,10 @@ async def main():
             args.max_pages_per_part
         )
 
+    total_term_extraction_total_tokens = 0
+    total_term_extraction_prompt_tokens = 0
+    total_term_extraction_completion_tokens = 0
+
     for file in pending_files:
         # 清理文件路径，去除两端的引号
         file = file.strip("\"'")
@@ -488,6 +635,7 @@ async def main():
             pages=args.pages,
             output_dir=args.output,
             translator=translator,
+            term_extraction_translator=term_extraction_translator,
             debug=args.debug,
             lang_in=args.lang_in,
             lang_out=args.lang_out,
@@ -522,8 +670,23 @@ async def main():
             primary_font_family=args.primary_font_family,
             only_include_translated_page=args.only_include_translated_page,
             save_auto_extracted_glossary=args.save_auto_extracted_glossary,
+            enable_graphic_element_process=not args.disable_graphic_element_process,
+            merge_alternating_line_numbers=args.merge_alternating_line_numbers,
+            skip_translation=args.skip_translation,
+            skip_form_render=args.skip_form_render,
+            skip_curve_render=args.skip_curve_render,
+            only_parse_generate_pdf=args.only_parse_generate_pdf,
+            remove_non_formula_lines=args.remove_non_formula_lines,
+            non_formula_line_iou_threshold=args.non_formula_line_iou_threshold,
+            figure_table_protection_threshold=args.figure_table_protection_threshold,
+            skip_formula_offset_calculation=args.skip_formula_offset_calculation,
+            metadata_extra_data=args.metadata_extra_data,
         )
 
+        def nop(_x):
+            pass
+
+        getattr(doc_layout_model, "init_font_mapper", nop)(config)
         # Create progress handler
         progress_context, progress_handler = create_progress_handler(config)
 
@@ -540,9 +703,26 @@ async def main():
                     result = event["translate_result"]
                     logger.info(str(result))
                     break
+        usage = config.term_extraction_token_usage
+        total_term_extraction_total_tokens += usage["total_tokens"]
+        total_term_extraction_prompt_tokens += usage["prompt_tokens"]
+        total_term_extraction_completion_tokens += usage["completion_tokens"]
     logger.info(f"Total tokens: {translator.token_count.value}")
     logger.info(f"Prompt tokens: {translator.prompt_token_count.value}")
     logger.info(f"Completion tokens: {translator.completion_token_count.value}")
+    logger.info(
+        "Term extraction tokens: total=%s prompt=%s completion=%s",
+        total_term_extraction_total_tokens,
+        total_term_extraction_prompt_tokens,
+        total_term_extraction_completion_tokens,
+    )
+    if term_extraction_translator is not translator:
+        logger.info(
+            "Term extraction translator raw tokens: total=%s prompt=%s completion=%s",
+            term_extraction_translator.token_count.value,
+            term_extraction_translator.prompt_token_count.value,
+            term_extraction_translator.completion_token_count.value,
+        )
 
 
 def create_progress_handler(translation_config: TranslationConfig):
